@@ -15,14 +15,15 @@ uniqueRef <- function(seqname, prefix) {
 }
 
 # XXX this needs to handle escaping special characters
-odfTableCaption <- function(caption, numformat='1', numlettersync=FALSE, formula='Table+1')
+odfTableCaption <- function(caption, numformat='1', numlettersync=FALSE,
+      formula='Table+1', label='Table')
 {
    if (!any(numformat == c('A', 'a', 'I', 'i', '1')))
       stop('illegal numformat value: ', numformat)
 
    refname <- uniqueRef('Table', 'refTable')
 
-   cat('<text:p text:style-name="Table">Table ')
+   cat(sprintf('<text:p text:style-name="Table">%s ', label))
    cat(sprintf('<text:sequence style:num-format="%s"', numformat))
    if (numlettersync)
       cat(' style:num-letter-sync="true"')
@@ -30,8 +31,7 @@ odfTableCaption <- function(caption, numformat='1', numlettersync=FALSE, formula
    cat(sprintf(' text:ref-name="%s">', refname))
    cat(sprintf('999</text:sequence>: %s</text:p>', caption))
    cat('\n')  # see if this removes the warning messages
-   invisible(NULL)
+
+   # return the reference name so it can be used to create a cross-reference
+   invisible(refname)
 }
-
-getRefs <- function() .odfEnv$seqInfo
-
