@@ -1,12 +1,14 @@
 odfWeaveControl <- function(
    zipCmd = c("zip -r $$file$$ .", "unzip -o $$file$$"),
-   cleanup = TRUE,
-   verbose = TRUE)
+   cleanup = ! debug,
+   verbose = TRUE,
+   debug = FALSE)
 {
 	# pass ... args to figGen here?
    list(
       zipCmd = zipCmd,
       cleanup = cleanup,
-      verbose = verbose)
+      verbose = verbose,
+      debug = debug)
 }
 
